// -*- C++ -*-

#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <string.h> 
#include <netdb.h> 
#include <sys/types.h> 
#include <netinet/in.h> 
#include <sys/socket.h> 
#include <sys/time.h>
#include <unistd.h>
#include "Mercator.h"
#include "NasAudio.h"

LOADABLE_CLASS_DEFN(NasAudio);

extern Mercator *mercator;

NasAudio::NasAudio():
  Audio("NasAudio"),
  serverFD(-1)
{
        struct hostent *he;
        struct sockaddr_in their_addr; /* connector's address information */

	/* Assuming server will be run on localhost */
	if(mercator->ExecList("nasaudioServer")) {
	    Error(EL_DEBUG, "NasAudio::NasAudio",
		  "nasaudioServer not found.", NULL);
	    return;
	}

	sleep(1);

        if ((he=gethostbyname("localhost")) == NULL) {  /* get the host info */
            herror("gethostbyname");
	    Error(EL_ERROR, "NasAudio::NasAudio",
		  "gethostbyname failed", NULL);
	    return;
        }

        if ((serverFD = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("socket");
	    Error(EL_ERROR, "NasAudio::NasAudio",
		  "could not create a socket", NULL);
	    return;
        }

        their_addr.sin_family = AF_INET;      /* host byte order */
        their_addr.sin_port = htons(SERVER_PORT);    /* short, network byte order */
        their_addr.sin_addr = *((struct in_addr *)he->h_addr);
        bzero(&(their_addr.sin_zero), 8);     /* zero the rest of the struct */

        if (connect(serverFD, (struct sockaddr *)&their_addr, \
                                              sizeof(struct sockaddr)) == -1) {
            perror("connect");
	    Error(EL_ERROR, "NasAudio::NasAudio",
		  "could not connect to server", NULL);
	    return;
        }
}

NasAudio::~NasAudio()
{
}

/*
  Currently, only the soundFile and volume parameters
  are used.
*/
SoundID
NasAudio::PlaySound(const char *soundFile, float *rate,
				  int volume, int muffle, int looped)
{
  char *ret;

  if(soundFile == NULL) {
    Error(EL_ERROR, "NasAudio::PlaySound",
	  "soundFile == NULL", NULL);
    return -1;
  }

  if(volume == 100) volume--;
  volume-=20;
  if(volume < 0) volume = 1;

  /* The difference between 40 and 90 in the loudness is too great.
     Normalizing... */

  volume /=2;
  volume += 10;

	sprintf(buf, "%d  ", NasStartSoundDirect);
	sprintf(buf+2, "%d  ", volume);
	sprintf(buf+4, "%s", soundFile);
	if (send(serverFD, buf, strlen(buf)+1, 0) == -1) {
                    perror("send");
		    Error(EL_ERROR, "NasAudio::PlaySound",
			  "couldn't send request to server", NULL);
		    return -1;
            }

	if(!waitForReply()) {
	  /* success - reply is in retBuf */
	  Error(EL_DEBUG, "NasAudio::PlaySound",
		"Played clip ", retBuf, ":", buf+2, NULL);
	  return atol(retBuf);
	} else
	  /* Error getting reply */
	  return (SoundID)-1;
}

int
NasAudio::waitForReply()
{
  fd_set rfds;
  struct timeval tv;
  int retval, numBytes;

  /* Watch serverFD to see when it has input. */
  FD_ZERO(&rfds);
  FD_SET(serverFD, &rfds);
  /* Wait up to 1 second. */
  tv.tv_sec = 1;
  tv.tv_usec = 0;

  retval = select(serverFD+1, &rfds, NULL, NULL, &tv);
  /* Don't rely on the value of tv now! */

  if (retval) {
  /* FD_ISSET(0, &rfds) will be true. */
    if((numBytes=recv(serverFD, retBuf, 32-1, 0)) > 0) {
      retBuf[numBytes] = '\n';
      return 0;
  } else
    return -1;
  }

  return -1; /* timeout - failure */
}



int
NasAudio::StopSound(SoundID soundID)
{
	sprintf(buf, "%d  ", NasStopSoundCached);
	sprintf(buf+2, "%ld  \n", soundID);
	if (send(serverFD, buf, strlen(buf)+1, 0) == -1) {
                    perror("send");
		    Error(EL_ERROR, "NasAudio::StopSound",
			  "couldn't send request to server", NULL);
		    return -1;
            }

	if(!waitForReply())
	  /* success - reply is in retBuf */
	  return atoi(retBuf);
	else
	  /* Error getting reply */
	  return -1;
}

int
NasAudio::StopAllSounds()
{
  return -1;
}

void
NasAudio::ShutDown()
{
  Error(EL_DEBUG, "NasAudio::ShutDown",
	"Shutting down NasAudio", NULL);

	sprintf(buf, "%d\n", NasQuit);
	if (send(serverFD, buf, strlen(buf)+1, 0) == -1) {
                    perror("send");
		    Error(EL_ERROR, "NasAudio::PlaySound",
			  "couldn't send request to server", NULL);
		    return;
            }

	if(waitForReply())
	  Error(EL_ERROR, "NasAudio::ShutDown",
		"Couldn't get reply from server for quit request", NULL);

	  return;
}
