/* A daemon which communicates with auvoxware server

When a client requests a clip be played, the clip's filename
is cached, its samples are loaded into a bucket at the auvoxware
server, and that bucket is played.  Future requests are looked up
in the local cache before being played.

Brian Sellden
*/

#ifndef _MNASD_H_
#define _MNAS_H_

#include <audio/audiolib.h>
#include <audio/soundlib.h>
#include <mDictValPtr.h>
#include <mDictValVal.h>
#include <mDictPtrPtr.h>
#include <mString.h>

#define LISTEN 0
#define CLIENT 1
#define LISTEN_PORT 3490
#define MAXDATASIZE 512
#define SUCCESS 0
#define FAILURE 1

typedef enum {
  NasLoadSound,
    NasStartSoundDirect,
  NasStartSoundCached,
    NasStopSoundDirect,
  NasStopSoundCached,
    NasStopAllSounds,
  NasQuit} NasRequest;

typedef struct {
  AuFlowID flow;
  AuDeviceID device; /* export device */
  int id;
} SoundStruct, *SoundStructPtr;

typedef mDictValPtr<int, SoundStruct> *IdDict;
typedef mDictPtrPtr<mString, SoundStruct> *SoundDict;

class mNasD {
    int initialized;
    int clientFD;
    int done;
    int currentID; 
    IdDict idDict;
    SoundDict soundDict;
    char buf[MAXDATASIZE];
    char *retBuf[2];

    /* auvoxware stuf */
    AuServer *aud;
    AuDeviceID monoDevice, stereoDevice;

 protected:
    int createFlowFromFile(const char*, SoundStructPtr);

 public:
 mNasD();
  mNasD(const char *servName, int new_fd);
  ~mNasD();

  int addFlow(int id, AuFlowID flow, mString sourceFile);
  int removeFlow(int id);

  void serveClient(int new_fd);
  int initialize(const char*);
  int ready();

  /*
  Handle requests
  */

  int handleRequest(int numBytes, char*result);
  int handleLoadRequest(int id, const char *fileName);
  int handleStartRequest(int id, int vol);
  int handleStartRequest(const char*);
  int handleStopRequest(int id);
  int handleStopRequest(const char*);
  int handleStopAllRequest();
  void handleQuitRequest();
};

#endif /* _MNASD_H_ */
